﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using ATAS.DataFeedsCore;
using ATAS.Indicators;
using ATAS.Indicators.Technical;
using ATAS.Strategies.Chart;
using TClusterSearch;

namespace TClusterSearchBot
{
    public class TClusterSearchStrategy : ChartStrategy
    {
        private int _lastBar;
        private readonly TClusterSearchIndicator _indCluster = new TClusterSearchIndicator();

        #region Public properties
        [Display(GroupName = "Signals & Patterns", Name = "Signals")]
        public ObservableCollection<TSignalData> SignalSettings
        {
            get => _indCluster.SignalSettings;
            set
            {
                _indCluster.SignalSettings = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Signals & Patterns", Name = "Patterns")]
        public ObservableCollection<TPattern> Patterns
        {
            get => _indCluster.Patterns;
            set
            {
                _indCluster.Patterns = value;
                RecalculateValues();
            }
        }

        private ObservableCollection<TSession> _sessions;
        [Display(GroupName = "Sessions", Name = "Sessions", Order = 13, Prompt = "1 value|{0} values|Add value...|Edit value...|Edit values...")]
        public ObservableCollection<TSession> Sessions
        {
            get => _indCluster.Sessions;
            set
            {
                _indCluster.Sessions = value;
                RecalculateValues();
            }
        }

        [Display(Name = "Enable", GroupName = "Sequence Settings")]
        public bool EnableSequence
        {
            get => _indCluster.EnableSequence;
            set
            {
                _indCluster.EnableSequence = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Position Size")]
        public int LotSize
        {
            get => _indCluster.LotSize;
            set
            {
                _indCluster.LotSize = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Take Profit")]
        public ObservableCollection<TTakeProfit> TakeProfits
        {
            get => _indCluster.TakeProfits;
            set
            {
                _indCluster.TakeProfits = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Stop Loss")]
        public int StopLossTicks
        {
            get => _indCluster.StopLossTicks;
            set
            {
                _indCluster.StopLossTicks = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Slippage")]
        public int Slippage
        {
            get => _indCluster.Slippage;
            set
            {
                _indCluster.Slippage = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Commission")]
        public decimal Comission
        {
            get => _indCluster.Comission;
            set
            {
                _indCluster.Comission = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Commission %")]
        public decimal ComissionPerc
        {
            get => _indCluster.ComissionPerc;
            set
            {
                _indCluster.ComissionPerc = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Tick Price")]
        public decimal TickPrice
        {
            get => _indCluster.TickPrice;
            set
            {
                _indCluster.TickPrice = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Increase Position")]
        public bool IncreasePosition
        {
            get => _indCluster.IncreasePosition;
            set
            {
                _indCluster.IncreasePosition = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Breakeven Ticks")]
        public int BreakevenPips
        {
            get => _indCluster.BreakevenPips;
            set
            {
                _indCluster.BreakevenPips = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Breakeven Plus")]
        public int BreakevenPlus
        {
            get => _indCluster.BreakevenPlus;
            set
            {
                _indCluster.BreakevenPlus = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Reverse By Close")]
        public bool EnableReverse
        {
            get => _indCluster.EnableReverse;
            set
            {
                _indCluster.EnableReverse = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Reverse By Stop")]
        public bool ReversePosition
        {
            get => _indCluster.ReversePosition;
            set
            {
                _indCluster.ReversePosition = value;
                RecalculateValues();
            }
        }

        [Display(GroupName = "Simulator", Name = "Maximal position")]
        public int MaxPosition
        {
            get => _indCluster.MaxPosition;
            set
            {
                _indCluster.MaxPosition = value;
                RecalculateValues();
            }
        }

        [Display(Name = "UseAlerts", GroupName = "Alerts")]
        public bool UseAlerts
        {
            get => _indCluster.UseAlerts;
            set
            {
                _indCluster.UseAlerts = value;
            }
        }

        [Display(Name = "AlertFile", GroupName = "Alerts")]
        public string AlertFile
        {
            get => _indCluster.AlertFile;
            set
            {
                _indCluster.AlertFile = value;
            }
        }

        [Display(Name = "Period", GroupName = "VWAP Settings")]
        public VWAP.VWAPPeriodType Type
        {
            get => _indCluster.Type;
            set
            {
                _indCluster.Type = value;
                RecalculateValues();
            }
        }

        [Display(Name = "Period", GroupName = "VWAP Settings")]
        public int Period
        {
            get => _indCluster.Period;
            set
            {
                _indCluster.Period = Math.Max(value, 1);
                RecalculateValues();
            }
        }

        [Display(Name = "FirstDev", GroupName = "VWAP Settings")]
        public decimal StDev
        {
            get => _indCluster.StDev;
            set
            {
                _indCluster.StDev = Math.Max(value, 0);
                RecalculateValues();
            }
        }

        [Display(Name = "SecondDev", GroupName = "VWAP Settings")]
        public decimal StDev1
        {
            get => _indCluster.StDev1;
            set
            {
                _indCluster.StDev1 = Math.Max(value, 0);
                RecalculateValues();
            }
        }

        [Display(Name = "ThirdDev", GroupName = "VWAP Settings")]
        public decimal StDev2
        {
            get => _indCluster.StDev2;
            set
            {
                _indCluster.StDev2 = Math.Max(value, 0);
                RecalculateValues();
            }
        }

        [Display(Name = "CustomSessionStart", GroupName = "VWAP Settings")]
        public TimeSpan CustomSessionStart
        {
            get => _indCluster.CustomSessionStart;
            set
            {
                _indCluster.CustomSessionStart = value;
                RecalculateValues();
            }
        }

        [Display(Name = "Days", GroupName = "VWAP Settings")]
        public int Days
        {
            get => _indCluster.Days;
            set
            {
                _indCluster.Days = value;
                RecalculateValues();
            }
        }
        #endregion Public properties

        public TClusterSearchStrategy()
        {
            _indCluster.EnableSimulator = false;
            _indCluster.LotSize = 5;
            _indCluster.StopLossTicks = 5;
            _indCluster.Slippage = 1;
            _indCluster.Comission = 1;
            _indCluster.TickPrice = 1;
            _indCluster.IncreasePosition = true;
            _indCluster.BreakevenPips = 10;
            _indCluster.BreakevenPlus = 5;
            _indCluster.EnableReverse = true;
            _indCluster.ReversePosition = false;
            _indCluster.MaxPosition = 3;

            DataSeries.Add(new ValueDataSeries("Up")
            {
                VisualType = VisualMode.UpArrow,
                ShowZeroValue = false,
                Color = Colors.Green,
                Width = 3
            });

            DataSeries.Add(new ValueDataSeries("Down")
            {
                VisualType = VisualMode.DownArrow,
                ShowZeroValue = false,
                Color = Colors.Red,
                Width = 3
            });
        }

        protected override void OnInitialize()
        {
            Add(_indCluster);
        }

        protected override void OnCalculate(int bar, decimal value)
        {
            var upSignal = ((ValueDataSeries)_indCluster.DataSeries[1])[bar];
            var dnSignal = ((ValueDataSeries)_indCluster.DataSeries[2])[bar];

            DataSeries[1][bar] = upSignal;
            DataSeries[2][bar] = dnSignal;

            if (!CanProcess(bar) || _lastBar == bar)
                return;
            
            CheckForClose();
            if (upSignal > 0)
            {
                if (!CheckReverse(OrderDirections.Sell))
                    return;

                if (!IncreasePosition && CurrentPosition > 0)
                    return;

                if (IncreasePosition && CurrentPosition + LotSize > MaxPosition)
                    return;

                OpenPosition(OrderDirections.Buy);
                _lastBar = bar;
            }
            if (dnSignal > 0)
            {
                if (!CheckReverse(OrderDirections.Buy))
                    return;

                if (!IncreasePosition && CurrentPosition < 0)
                    return;

                if (IncreasePosition && Math.Abs(CurrentPosition) + LotSize > MaxPosition)
                    return;

                OpenPosition(OrderDirections.Sell);
                _lastBar = bar;
            }
        }

        protected override void OnCurrentPositionChanged()
        {
            Process();
        }

        private void Process()
        {
            CancelAllOrders();

            if (CurrentPosition == 0)
                return;

            var tp = TakeProfits.Where(t => t.Type == TakeProfitType.Ticks).FirstOrDefault();
            if (tp != null)
            {
                var take = new Order
                {
                    Security = Security,
                    Portfolio = Portfolio,
                    Type = ATAS.DataFeedsCore.OrderTypes.Limit,
                    Direction = CurrentPosition > 0 ? OrderDirections.Sell : OrderDirections.Buy,
                    Price = AveragePrice + tp.TicksCount * Security.TickSize * (CurrentPosition > 0 ? 1 : -1),
                    QuantityToFill = Math.Abs(CurrentPosition),
                    Comment = "TP",
                };

                OpenOrder(take);
            }

            var stop = new Order
            {
                Security = Security,
                Portfolio = Portfolio,
                Type = ATAS.DataFeedsCore.OrderTypes.Stop,
                Direction = CurrentPosition > 0 ? OrderDirections.Sell : OrderDirections.Buy,
                TriggerPrice = AveragePrice - StopLossTicks * Security.TickSize * (CurrentPosition > 0 ? 1 : -1),
                QuantityToFill = Math.Abs(CurrentPosition),
                Comment = "SP",
            };

            OpenOrder(stop);
        }

        private void OpenPosition(OrderDirections direction)
        {
            var order = new Order
            {
                Portfolio = Portfolio,
                Security = Security,
                Direction = direction,
                Type = ATAS.DataFeedsCore.OrderTypes.Market,
                QuantityToFill = LotSize,
            };

            OpenOrder(order);
        }

        private void CloseCurrentPosition()
        {
            var order = new Order
            {
                Portfolio = Portfolio,
                Security = Security,
                Direction = CurrentPosition > 0 ? OrderDirections.Sell : OrderDirections.Buy,
                Type = ATAS.DataFeedsCore.OrderTypes.Market,
                QuantityToFill = Math.Abs(CurrentPosition),
            };

            OpenOrder(order);
        }

        private bool CheckReverse(OrderDirections direction)
        {
            if (CurrentPosition == 0)
                return (true);
            if (CurrentPosition > 0 && direction == OrderDirections.Buy)
                return (true);
            if (CurrentPosition < 0 && direction == OrderDirections.Sell)
                return (true);

            if (!TakeProfits.Any(q => q.Type == TakeProfitType.Reverse))
                return (false);

            CloseCurrentPosition();
            return (EnableReverse);
        }

        private void CheckForClose()
        {
            if (CurrentPosition == 0)
                return;
            int sign = CurrentPosition > 0 ? -1 : 1;
        }

        private void CancelAllOrders()
        {
            foreach (var order in Orders.Where(t => t.State == OrderStates.Active))
            {
                CancelOrder(order);
            }
        }
    }
}
